/**********************************************************************



  Filename:  vec.cpp

  Chapter:   7      Visitation Iterators and Containers

  Section:   7.7    Numerical Integration using vector

  Compiler:  Borland C++     Version 5.0       Summer 1996

  C++ for Fortran Programmers    By Ira Pohl



********************************************************************/



//vec.h

#include <iostream.h>

#include <assert.h>

class vector {

public:

   typedef double* iterator; //pointer vector::double*

   explicit vector(int n = 1):size(n)

      {assert(n > 0); bp = new double[n];  assert(bp != 0);}

   vector(const vector& v):size(v.size)

      {bp = new double[size]; assert(bp != 0);

      for(int i = 0; i < size; ++i)

          bp[i] = (v.bp)[i];

      }

   vector(const double a[], int n):size(n)

      {assert(n > 0); bp = new double[n];  assert(bp != 0);

       for(int i = 0; i < size; ++i)

          bp[i] = a[i];               }

   ~vector() { delete []bp; }

   iterator begin()const {return bp;}

   iterator end()const {return bp + size;}

   int  ub()const {return size - 1;}//upper bound

   double& operator[](int i); //range checked

   vector& operator=(const vector& v);

private:

   iterator bp; //base pointer

   int size; //number of elements

};

 double& vector::operator[](int i)

 {

    assert((i >= 0) && (i < size));

    return bp[i];

 }

vector& vector::operator=(const vector& v)

{

      if (bp != v.bp){

         iterator temp = bp;

          bp = new double[v.size]; assert(bp != 0);

          for(int i = 0; i < size; ++i)

             bp[i] = (v.bp)[i];

          delete []temp;

      }

      return *this;

}



ostream& operator<<(ostream& out, const vector& v)

{

   vector::iterator p;



   for (p = v.begin(); p != v.end(); ++p)

      out << *p << '\t';

   out << endl;

   return out;

}



istream& operator>>(istream& in, vector& v)

{

   vector::iterator p;



   for (p = v.begin(); p != v.end(); ++p)

      in >> *p;

   return in;

}

inline double f(double x) { return 3 * x * x + 4* x + 1;}

//two roots  (x+1)(3x +1)

inline double abs(double x){ return (x>0 ? x : -x);}

vector::iterator find_zero(const vector& v)

{

    vector::iterator u = v.begin(), t = u;

    double eps = abs(*t);



    while(++t != v.end())

    {

       if (abs(*t)< eps) { u = t; eps = abs(*t);}

    }

    return u;

}

double bisect(double f(double), double a, double b, double eps)

{

  //cout <<"enter bisect" << a << " , " << b << endl;

  if( (b - a) < eps)

     return ( (b + a) / 2);

  else

     if ( f((b+a)/2) <= 0)

        return  bisect(f, ( b+a )/ 2, b, eps);

     else

        return bisect(f, a, ( b+a )/ 2,  eps);

}



int main()

{

 cout << "hello world" << endl;

 vector y(1000);

 double x = -2, incr = 2.0/y.ub();

 for( int i = 0; i <= y.ub(); ++i) {

   y[i] = f(x); x += incr;

 }

 // cout << y << endl;

  cout << "residual of y = " <<*find_zero(y) << endl;

   //change in adjacent position or smallest residual

   //depends on which criteria remains true?

  cout << "zero at = "

     << ( -2.0 + incr *  static_cast<int>( find_zero(y) - y.begin())) << endl;

  cout << " root using bisection "

     <<   bisect(f, -2.0, 0, 0.0000001) << endl;

  int xxxx; cin >>xxxx;

}